/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { MappingProperty } from '@elastic/elasticsearch/lib/api/types';

/**
 * These `mappingsProperties` represent mappings that were generated by
 * Elasticsearch automatically, for an index named `auditbeat-custom-index-1`:
 *
 * ```
 * DELETE auditbeat-custom-index-1
 *
 * PUT auditbeat-custom-index-1
 *
 * PUT auditbeat-custom-index-1/_mapping
 * {
 *   "properties": {
 *     "@timestamp": {
 *       "type": "date"
 *     },
 *     "event.category": {
 *       "type": "keyword",
 *       "ignore_above": 1024
 *     }
 *   }
 * }
 * ```
 *
 * when the following document was inserted:
 *
 * ```
 * POST auditbeat-custom-index-1/_doc
 * {
 *   "@timestamp": "2023-02-06T09:41:49.668Z",
 *   "host": {
 *     "name": "foo"
 *   },
 *   "event": {
 *     "category": "an_invalid_category"
 *   },
 *   "some.field": "this",
 *   "source": {
 *     "port": 90210,
 *     "ip": "10.1.2.3"
 *   }
 * }
 * ```
 */
export const mockMappingsProperties: Record<string, MappingProperty> = {
  '@timestamp': {
    type: 'date',
  },
  event: {
    properties: {
      category: {
        type: 'keyword',
        ignore_above: 1024,
      },
    },
  },
  host: {
    properties: {
      name: {
        type: 'text',
        fields: {
          keyword: {
            type: 'keyword',
            ignore_above: 256,
          },
        },
      },
    },
  },
  some: {
    properties: {
      field: {
        type: 'text',
        fields: {
          keyword: {
            type: 'keyword',
            ignore_above: 256,
          },
        },
      },
    },
  },
  source: {
    properties: {
      ip: {
        type: 'text',
        fields: {
          keyword: {
            type: 'keyword',
            ignore_above: 256,
          },
        },
      },
      port: {
        type: 'long',
      },
    },
  },
};
