/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import type { TypeOf } from '@kbn/config-schema';
import type { EcsError } from '@elastic/ecs';
import type { BaseFileMetadata, FileCompression, FileJSON } from '@kbn/files-plugin/common';
import type {
  UploadActionApiRequestBody,
  ActionStatusRequestSchema,
  KillProcessRequestBody,
  SuspendProcessRequestBody,
} from '../../api/endpoint';

import type {
  ResponseActionAgentType,
  ResponseActionsApiCommandNames,
  ResponseActionStatus,
} from '../service/response_actions/constants';

export type ISOLATION_ACTIONS = 'isolate' | 'unisolate';

/** The output provided by some of the Endpoint responses */
export interface ActionResponseOutput<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput
> {
  type: 'json' | 'text';
  content: TOutputContent;
}

export interface ProcessesEntry {
  command: string;
  pid: string;
  entity_id: string;
  user: string;
}

export interface GetProcessesActionOutputContent {
  code: string;
  entries: ProcessesEntry[];
}

export interface SuspendProcessActionOutputContent {
  code: string;
  command?: string;
  pid?: number;
  entity_id?: string;
}

export interface KillProcessActionOutputContent {
  code: string;
  command?: string;
  pid?: number;
  entity_id?: string;
  process_name?: string;
}

export interface ResponseActionGetFileOutputContent {
  code: string;
  zip_size: number;
  /** The contents of the zip file. One entry per file */
  contents: Array<{
    path: string;
    sha256: string;
    size: number;
    file_name: string;
    type: string;
  }>;
}

export interface ResponseActionExecuteOutputContent {
  code: string;
  /* The truncated 'tail' output of the command */
  stdout: string;
  /* The truncated 'tail' of any errors generated by the command */
  stderr: string;
  stdout_truncated: boolean;
  stderr_truncated: boolean;
  shell_code: number;
  /* The shell used to execute the command */
  shell: string;
  /* The current working directory used when the command was executed */
  cwd: string;
  output_file_id: string;
  /** Informs whether the  stdout/stderr files are
   * truncated due to size limitations (50 Mb each)
   * */
  output_file_stdout_truncated: boolean;
  output_file_stderr_truncated: boolean;
}

export interface ResponseActionScanOutputContent {
  code: string;
}

export interface ResponseActionRunScriptOutputContent {
  stdout: string;
  stderr: string;
  code: string;
}

export const ActivityLogItemTypes = {
  ACTION: 'action' as const,
  RESPONSE: 'response' as const,
  FLEET_ACTION: 'fleetAction' as const,
  FLEET_RESPONSE: 'fleetResponse' as const,
};

interface EndpointActionFields<
  TParameters extends EndpointActionDataParameterTypes = EndpointActionDataParameterTypes,
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput
> {
  action_id: string;
  data: EndpointActionData<TParameters, TOutputContent>;
}

interface ActionRequestFields {
  expiration: string;
  type: 'INPUT_ACTION';
  input_type: ResponseActionAgentType;
}

interface ActionResponseFields {
  completed_at: string;
  started_at: string;
}

/**
 * An endpoint Action created in the Endpoint's `.logs-endpoint.actions-default` index.
 * @since v7.16
 */
export interface LogsEndpointAction<
  TParameters extends EndpointActionDataParameterTypes = EndpointActionDataParameterTypes,
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput,
  TMeta extends {} = {}
> {
  '@timestamp': string;
  agent: {
    id: string | string[];
  };
  EndpointActions: EndpointActionFields<TParameters, TOutputContent> & ActionRequestFields;
  error?: EcsError;
  user: {
    id: string;
  };
  rule?: {
    id: string;
    name: string;
  };
  /** Area to store any additional metadata  */
  meta?: TMeta;
}

export interface LogsEndpointActionWithHosts extends LogsEndpointAction {
  EndpointActions: EndpointActionFields &
    ActionRequestFields & {
      data: EndpointActionData & {
        hosts: Record<string, { name: string }>;
      };
    };
}

/**
 * An Action response written by the endpoint to the Endpoint `.logs-endpoint.action.responses` datastream
 * @since v7.16
 */
export interface LogsEndpointActionResponse<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput,
  TMeta extends {} = {}
> {
  '@timestamp': string;
  agent: {
    id: string | string[];
  };
  EndpointActions: ActionResponseFields & {
    action_id: string;
    // Endpoint Response documents do not have `parameters` in the `data`
    data: Pick<EndpointActionData<never, TOutputContent>, 'comment' | 'command' | 'output'>;
    input_type: ResponseActionAgentType;
  };
  error?: EcsError;
  /** Area to store any other metadata for the action response */
  meta?: TMeta;
}

export interface ResponseActionParametersWithPid {
  pid: number;
  entity_id?: never;
  process_name?: never;
}

export interface ResponseActionParametersWithEntityId {
  pid?: never;
  process_name?: never;
  entity_id: string;
}

export interface ResponseActionParametersWithProcessName {
  pid?: never;
  entity_id?: never;
  process_name: string;
}

export type ResponseActionParametersWithProcessData =
  | ResponseActionParametersWithPid
  | ResponseActionParametersWithEntityId
  | ResponseActionParametersWithProcessName;

export interface ResponseActionGetFileParameters {
  path: string;
}

export interface ResponseActionsExecuteParameters {
  command: string;
  timeout?: number;
}

export interface ResponseActionScanParameters {
  path: string;
}

// Currently reflecting CrowdStrike's RunScript parameters
interface ActionsRunScriptParametersBase {
  Raw?: string;
  HostPath?: string;
  CloudFile?: string;
  CommandLine?: string;
  Timeout?: number;
}

// Enforce at least one of the script parameters is required
export type ResponseActionRunScriptParameters = AtLeastOne<
  ActionsRunScriptParametersBase,
  'Raw' | 'HostPath' | 'CloudFile'
>;

export type EndpointActionDataParameterTypes =
  | undefined
  | ResponseActionParametersWithProcessData
  | ResponseActionsExecuteParameters
  | ResponseActionGetFileParameters
  | ResponseActionUploadParameters
  | ResponseActionScanParameters
  | ResponseActionRunScriptParameters;

/** Output content of the different response actions */
export type EndpointActionResponseDataOutput =
  | Record<string, never> // Empty object
  | ResponseActionExecuteOutputContent
  | ResponseActionGetFileOutputContent
  | ResponseActionUploadOutputContent
  | GetProcessesActionOutputContent
  | SuspendProcessActionOutputContent
  | KillProcessActionOutputContent
  | ResponseActionScanOutputContent
  | ResponseActionRunScriptOutputContent;

/**
 * The data stored with each Response Action under `EndpointActions.data` property
 */
export interface EndpointActionData<
  TParameters extends EndpointActionDataParameterTypes = EndpointActionDataParameterTypes,
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput
> {
  command: ResponseActionsApiCommandNames;
  comment?: string;
  parameters?: TParameters;
  output?: ActionResponseOutput<TOutputContent>;
  /**
   * If `alert_id` is defined, then action request is of type `automated`
   *
   * **IMPORTANT**: should be used only when response actions are created from a Rule (automated response actions)
   *                as this property is used to determine if an action is of type `automated`
   */
  alert_id?: string[];
  hosts?: Record<string, { name: string }>;
}

export interface FleetActionResponseData {
  endpoint?: {
    ack?: boolean;
  };
}

/**
 * And endpoint action created in Fleet's `.fleet-actions`
 */
export interface EndpointAction extends ActionRequestFields {
  action_id: string;
  '@timestamp': string;
  agents: string[];
  user_id: string;
  // the number of seconds Elastic Agent (on the host) should
  // wait to send back an action result before it will timeout
  timeout?: number;
  data: EndpointActionData;
  // signature of the endpoint action
  signed?: {
    data: string;
    signature: string;
  };
}

/**
 * The action response created in Fleet's index after the action has been successfully delivered to
 * the endpoint
 */
export interface EndpointActionResponse {
  '@timestamp': string;
  /** The id of the action for which this response is associated with */
  action_id: string;
  /** The agent id that sent this action response */
  agent_id: string;
  /** timestamp when the action started to be processed by the Elastic Agent and/or Endpoint on the host */
  started_at: string;
  /** timestamp when the action completed processing by the Elastic Agent and/or Endpoint on the host */
  completed_at: string;
  error?: string;
  action_data: EndpointActionData;
  /* Response data from the Endpoint process -- only present in 7.16+ */
  action_response?: FleetActionResponseData;
}

export interface EndpointActivityLogAction {
  type: typeof ActivityLogItemTypes.ACTION;
  item: {
    id: string;
    data: LogsEndpointAction;
  };
}

export interface EndpointActivityLogActionResponse<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput
> {
  type: typeof ActivityLogItemTypes.RESPONSE;
  item: {
    id: string;
    data: LogsEndpointActionResponse<TOutputContent>;
  };
}

export interface ActivityLogAction {
  type: typeof ActivityLogItemTypes.FLEET_ACTION;
  item: {
    // document _id
    id: string;
    // document _source
    data: EndpointAction;
  };
}

export interface ActivityLogActionResponse {
  type: typeof ActivityLogItemTypes.FLEET_RESPONSE;
  item: {
    // document id
    id: string;
    // document _source
    data: EndpointActionResponse;
  };
}

/**
 * One of the possible Response Action Log entry - Either a Fleet Action request, Fleet action response,
 * Endpoint action request and/or endpoint action response.
 */
export type ActivityLogEntry =
  | ActivityLogAction
  | ActivityLogActionResponse
  | EndpointActivityLogAction
  | EndpointActivityLogActionResponse;

export interface ActivityLog {
  page: number;
  pageSize: number;
  startDate: string;
  endDate: string;
  data: ActivityLogEntry[];
}

/** Note: this type should almost never be used. Use instead the response action specific types above */
export type KillOrSuspendProcessRequestBody = KillProcessRequestBody & SuspendProcessRequestBody;

export interface HostIsolationResponse {
  action: string;
}

export interface ResponseActionApiResponse<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput
> {
  action?: string;
  data: ActionDetails<TOutputContent>;
}

export interface EndpointPendingActions {
  agent_id: string;
  /** Number of actions pending for each type */
  pending_actions: Partial<Record<ResponseActionsApiCommandNames, number>> & {
    // Defined any other key just in case we get back some other actions
    [key: string]: number;
  };
}

export interface PendingActionsResponse {
  data: EndpointPendingActions[];
}

export type PendingActionsRequestQuery = TypeOf<typeof ActionStatusRequestSchema.query>;

export interface ActionDetailsAgentState {
  isCompleted: boolean;
  wasSuccessful: boolean;
  errors: undefined | string[];
  completedAt: string | undefined;
}

export interface ActionDetails<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput,
  TParameters extends EndpointActionDataParameterTypes = EndpointActionDataParameterTypes
> {
  /**
   * The action ID. This is a legacy property action and should no longer be used. Only here for
   * backwards compatibility
   *
   * @deprecated
   */
  action?: string;

  /** The action id */
  id: string;
  /**
   * The Endpoint ID (and fleet agent ID - they are the same) for which the action was created for.
   * This is an Array because the action could have been sent to multiple endpoints.
   */
  agents: string[];
  /**
   * A map of `Agent ID`'s to which the action was sent whose value contains more
   * information about the host (currently the host name only).
   */
  hosts: Record<string, { name: string }>;
  /**
   * The Endpoint type of action (ex. `isolate`, `release`) that is being requested to be
   * performed on the endpoint
   */
  command: ResponseActionsApiCommandNames;
  /**
   * Will be set to true only if action is not yet completed and elapsed time has exceeded
   * the request's expiration date
   */
  isExpired: boolean;
  /** Action has been completed */
  isCompleted: boolean;
  /** If the action was successful */
  wasSuccessful: boolean;
  /** Any errors encountered if `wasSuccessful` is `false` */
  errors: undefined | string[];
  /** The date when the initial action request was submitted */
  startedAt: string;
  /** The date when the action was completed (a response by the endpoint (not fleet) was received) */
  completedAt: string | undefined;
  /** The output data from an action stored in an object where the key is the agent id */
  outputs?: Record<string, ActionResponseOutput<TOutputContent>>;
  /**
   * A map by Agent ID holding information about the action for the specific agent.
   * Helpful when action is sent to multiple agents
   */
  agentState: Record<string, ActionDetailsAgentState>;
  /**  action status */
  status: ResponseActionStatus;
  /** user that created the action */
  createdBy: string;
  /** comment submitted with action */
  comment?: string;
  /** parameters submitted with action */
  parameters?: TParameters;
  alertIds?: string[];
  ruleId?: string;
  ruleName?: string;
  /** The agent type to where the response action was sent */
  agentType: ResponseActionAgentType;
}

export interface ActionDetailsApiResponse<
  TOutputContent extends EndpointActionResponseDataOutput = EndpointActionResponseDataOutput,
  TParameters extends EndpointActionDataParameterTypes = EndpointActionDataParameterTypes
> {
  data: ActionDetails<TOutputContent, TParameters>;
}

/** Action Details normally returned by Action List API response  */
export type ActionDetailsNoOutputs = Omit<ActionDetails, 'outputs'>;

export interface ActionListApiResponse {
  page: number | undefined;
  pageSize: number | undefined;
  startDate: string | undefined;
  agentTypes: ResponseActionAgentType[] | undefined;
  elasticAgentIds: string[] | undefined;
  endDate: string | undefined;
  userIds: string[] | undefined; // users that requested the actions
  commands: string[] | undefined; // type of actions
  /**
   * Introduced in 8.8, outputs are visible for specific actions like `execute`.
   * The `outputs` are part the list response for given set of actions.
   * `outputs` for all actions are restricted due to possibly large amounts of
   * data, especially for cases (in the future) where we might support actions being sent to
   * multiple agents
   */
  data: Array<ActionDetails | ActionDetailsNoOutputs>;
  statuses: ResponseActionStatus[] | undefined;
  total: number;
}

/**
 * File upload metadata information. Stored by endpoint/fleet-server when a file is uploaded to ES in connection with
 * a response action
 */
export interface FileUploadMetadata {
  action_id: string;
  agent_id: string;
  src: string; // The agent name. `endpoint` for security solution files
  upload_id: string;
  upload_start: number;
  contents: Array<
    Partial<{
      accessed: string; // ISO date
      created: string; // ISO date
      directory: string;
      file_extension: string;
      file_name: string;
      gid: number;
      inode: number;
      mode: string;
      mountpoint: string;
      mtime: string;
      path: string;
      sha256: string;
      size: number;
      target_path: string;
      type: string;
      uid: number;
    }>
  >;
  file: Pick<
    Required<BaseFileMetadata>,
    'name' | 'size' | 'Status' | 'ChunkSize' | 'mime_type' | 'extension'
  > &
    Omit<BaseFileMetadata, 'name' | 'size' | 'Status' | 'ChunkSize' | 'mime_type' | 'extension'> & {
      compression: FileCompression;
      attributes: string[];
      type: string;
    };
  host: {
    hostname: string;
  };
  transithash: {
    sha256: string;
  };
  '@timestamp': string;
}

export type UploadedFileInfo = Pick<
  FileJSON,
  'name' | 'id' | 'mimeType' | 'size' | 'status' | 'created'
> & {
  actionId: string;
  agentId: string;
  agentType: ResponseActionAgentType;
};

export interface ActionFileInfoApiResponse {
  data: UploadedFileInfo;
}

/**
 * The parameters that are sent to the Endpoint.
 *
 * NOTE: Most of the parameters below are NOT accepted via the API. They are inserted into
 * the action's parameters via the API route handler
 */
export type ResponseActionUploadParameters = UploadActionApiRequestBody['parameters'] & {
  file_sha256: string;
  file_size: number;
  file_name: string;
  file_id: string;
};

export interface ResponseActionUploadOutputContent {
  code: string;
  /** Full path to the file on the host machine where it was saved */
  path: string;
  /** The free space available (after saving the file) of the drive where the file was saved to, In Bytes  */
  disk_free_space: number;
}

type AtLeastOne<T, K extends keyof T = keyof T> = K extends keyof T
  ? Required<Pick<T, K>> & Partial<Omit<T, K>>
  : never;
