/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React from 'react';
import { act, fireEvent, render } from '@testing-library/react';
import { mountWithIntl } from '@kbn/test-jest-helpers';
import { EuiTextArea } from '@elastic/eui';

import { ExceptionItemComments } from '.';
import { TestProviders } from '../../../../common/mock';
import { useCurrentUser } from '../../../../common/lib/kibana';
import { shallow } from 'enzyme';
import { MAX_COMMENT_LENGTH } from '../../../../../common/constants';

jest.mock('../../../../common/lib/kibana');

describe('ExceptionItemComments', () => {
  beforeEach(() => {
    (useCurrentUser as jest.Mock).mockReturnValue({
      username: 'user',
      email: 'email',
      fullName: 'full name',
      roles: ['user-role'],
      enabled: true,
      authentication_realm: { name: 'native1', type: 'native' },
      lookup_realm: { name: 'native1', type: 'native' },
      authentication_provider: { type: 'basic', name: 'basic1' },
      authentication_type: 'realm',
      elastic_cloud_user: false,
      metadata: { _reserved: false },
    });
  });

  it('it uses user full_name if one exists', () => {
    const wrapper = shallow(
      <ExceptionItemComments
        newCommentValue={'This is a new comment'}
        newCommentOnChange={jest.fn()}
        setCommentError={jest.fn()}
      />
    );

    expect(wrapper.find('[data-test-subj="exceptionItemCommentAvatar"]').prop('name')).toEqual(
      'full name'
    );
  });

  it('it uses user email if fullName is not available', () => {
    (useCurrentUser as jest.Mock).mockReturnValue({
      username: 'user',
      email: 'email',
      fullName: '',
      roles: ['user-role'],
      enabled: true,
      authentication_realm: { name: 'native1', type: 'native' },
      lookup_realm: { name: 'native1', type: 'native' },
      authentication_provider: { type: 'basic', name: 'basic1' },
      authentication_type: 'realm',
      elastic_cloud_user: false,
      metadata: { _reserved: false },
    });

    const wrapper = shallow(
      <ExceptionItemComments
        newCommentValue={'This is a new comment'}
        newCommentOnChange={jest.fn()}
        setCommentError={jest.fn()}
      />
    );

    expect(wrapper.find('[data-test-subj="exceptionItemCommentAvatar"]').prop('name')).toEqual(
      'email'
    );
  });

  it('it uses username if fullName and email are not available', () => {
    (useCurrentUser as jest.Mock).mockReturnValue({
      username: 'user',
      email: '',
      fullName: '',
      roles: ['user-role'],
      enabled: true,
      authentication_realm: { name: 'native1', type: 'native' },
      lookup_realm: { name: 'native1', type: 'native' },
      authentication_provider: { type: 'basic', name: 'basic1' },
      authentication_type: 'realm',
      elastic_cloud_user: false,
      metadata: { _reserved: false },
    });

    const wrapper = shallow(
      <ExceptionItemComments
        newCommentValue={'This is a new comment'}
        newCommentOnChange={jest.fn()}
        setCommentError={jest.fn()}
      />
    );

    expect(wrapper.find('[data-test-subj="exceptionItemCommentAvatar"]').prop('name')).toEqual(
      'user'
    );
  });

  it('it renders new comment', () => {
    const wrapper = mountWithIntl(
      <TestProviders>
        <ExceptionItemComments
          newCommentValue={'This is a new comment'}
          newCommentOnChange={jest.fn()}
          setCommentError={jest.fn()}
        />
      </TestProviders>
    );

    expect(
      wrapper.find('[data-test-subj="newExceptionItemCommentTextArea"]').at(1).props().value
    ).toEqual('This is a new comment');
  });

  it('it calls newCommentOnChange on comment update change', () => {
    const mockOnCommentChange = jest.fn();
    const wrapper = mountWithIntl(
      <TestProviders>
        <ExceptionItemComments
          newCommentValue="This is a new comment"
          newCommentOnChange={mockOnCommentChange}
          setCommentError={jest.fn()}
        />
      </TestProviders>
    );

    (
      wrapper.find(EuiTextArea).at(0).props() as unknown as {
        onChange: (e: React.ChangeEvent<HTMLInputElement>) => void;
      }
    ).onChange({
      target: { value: 'Updating my new comment' },
    } as React.ChangeEvent<HTMLInputElement>);

    expect(mockOnCommentChange).toHaveBeenCalledWith('Updating my new comment');
  });

  it('it renders existing comments if any exist', () => {
    const mockOnCommentChange = jest.fn();
    const wrapper = mountWithIntl(
      <TestProviders>
        <ExceptionItemComments
          exceptionItemComments={[
            {
              comment: 'This is an existing comment',
              created_at: '2020-04-20T15:25:31.830Z',
              created_by: 'elastic',
              id: 'uuid_here',
            },
          ]}
          newCommentValue={''}
          newCommentOnChange={mockOnCommentChange}
          setCommentError={jest.fn()}
        />
      </TestProviders>
    );

    expect(wrapper.find('[data-test-subj="exceptionItemCommentsAccordion"]').exists()).toBeTruthy();
  });

  it('it calls setCommentError on comment error update change', async () => {
    const mockSetCommentError = jest.fn();
    const { getByLabelText, queryByText } = render(
      <TestProviders>
        <ExceptionItemComments
          newCommentValue="This is a new comment"
          newCommentOnChange={jest.fn()}
          setCommentError={mockSetCommentError}
        />
      </TestProviders>
    );

    const commentInput = getByLabelText('Comment Input');

    const commentErrorMessage = `The length of the comment is too long. The maximum length is ${MAX_COMMENT_LENGTH} characters.`;
    expect(queryByText(commentErrorMessage)).toBeNull();

    // Put comment with the length above maximum allowed
    act(() => {
      fireEvent.change(commentInput, {
        target: {
          value: [...new Array(MAX_COMMENT_LENGTH + 1).keys()].map((_) => 'a').join(''),
        },
      });
      fireEvent.blur(commentInput);
    });
    expect(queryByText(commentErrorMessage)).not.toBeNull();
    expect(mockSetCommentError).toHaveBeenCalledWith(true);

    // Put comment with the allowed length
    act(() => {
      fireEvent.change(commentInput, {
        target: {
          value: 'Updating my new comment',
        },
      });
      fireEvent.blur(commentInput);
    });
    expect(queryByText(commentErrorMessage)).toBeNull();
    expect(mockSetCommentError).toHaveBeenCalledWith(false);
  });
});
