/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import type { RenderHookResult } from '@testing-library/react';
import { waitFor, renderHook } from '@testing-library/react';

import { coreMock } from '@kbn/core/public/mocks';
import * as rulesApi from '../../rule_management/api/api';
import * as listsApi from '@kbn/securitysolution-list-api';
import { getExceptionListSchemaMock } from '@kbn/lists-plugin/common/schemas/response/exception_list_schema.mock';
import { savedRuleMock } from '../../rule_management/logic/mock';
import type {
  ExceptionListType,
  ListArray,
  ExceptionListSchema,
} from '@kbn/securitysolution-io-ts-list-types';
import { getListArrayMock } from '../../../../common/detection_engine/schemas/types/lists.mock';
import type {
  UseFetchOrCreateRuleExceptionListProps,
  ReturnUseFetchOrCreateRuleExceptionList,
} from './use_fetch_or_create_rule_exception_list';
import { useFetchOrCreateRuleExceptionList } from './use_fetch_or_create_rule_exception_list';

const mockKibanaHttpService = coreMock.createStart().http;
jest.mock('../../rule_management/api/api');
jest.mock('@kbn/securitysolution-list-api');

describe('useFetchOrCreateRuleExceptionList', () => {
  let fetchRuleById: jest.SpyInstance<ReturnType<typeof rulesApi.fetchRuleById>>;
  let patchRule: jest.SpyInstance<ReturnType<typeof rulesApi.patchRule>>;
  let addExceptionList: jest.SpyInstance<ReturnType<typeof listsApi.addExceptionList>>;
  let addEndpointExceptionList: jest.SpyInstance<
    ReturnType<typeof listsApi.addEndpointExceptionList>
  >;
  let fetchExceptionListById: jest.SpyInstance<Promise<ExceptionListSchema>>;
  let render: (
    listType?: UseFetchOrCreateRuleExceptionListProps['exceptionListType']
  ) => RenderHookResult<
    ReturnUseFetchOrCreateRuleExceptionList,
    UseFetchOrCreateRuleExceptionListProps
  >;
  const onError = jest.fn();
  const onSuccess = jest.fn();
  const error = new Error('Something went wrong');
  const ruleId = 'myRuleId';
  const abortCtrl = new AbortController();
  const detectionListType: ExceptionListType = 'detection';
  const endpointListType: ExceptionListType = 'endpoint';
  const detectionExceptionList = {
    ...getExceptionListSchemaMock(),
    type: detectionListType,
  };
  const endpointExceptionList = {
    ...getExceptionListSchemaMock(),
    type: endpointListType,
  };
  const newDetectionExceptionList = {
    ...detectionExceptionList,
    name: 'new detection exception list',
  };
  const newEndpointExceptionList = {
    ...endpointExceptionList,
    name: 'new endpoint exception list',
  };
  const exceptionsListReferences: ListArray = getListArrayMock();
  const ruleWithExceptionLists = {
    ...savedRuleMock,
    exceptions_list: exceptionsListReferences,
  };
  const ruleWithoutExceptionLists = {
    ...savedRuleMock,
    exceptions_list: [],
  };

  beforeEach(() => {
    fetchRuleById = jest.spyOn(rulesApi, 'fetchRuleById').mockResolvedValue(ruleWithExceptionLists);

    patchRule = jest.spyOn(rulesApi, 'patchRule');

    addExceptionList = jest
      .spyOn(listsApi, 'addExceptionList')
      .mockResolvedValue(newDetectionExceptionList);

    addEndpointExceptionList = jest
      .spyOn(listsApi, 'addEndpointExceptionList')
      .mockResolvedValue(newEndpointExceptionList);

    fetchExceptionListById = jest
      .spyOn(listsApi, 'fetchExceptionListById')
      .mockResolvedValue(detectionExceptionList);

    render = (listType = detectionListType) =>
      renderHook(() =>
        useFetchOrCreateRuleExceptionList({
          http: mockKibanaHttpService,
          ruleId,
          exceptionListType: listType,
          onError,
          onSuccess,
        })
      );
  });

  afterEach(() => {
    jest.clearAllMocks();
  });

  it('initializes hook', async () => {
    const { result } = render();

    // Should set isLoading to true while fetching
    expect(result.current).toEqual([true, null]);

    await waitFor(() => expect(result.current).toEqual([false, detectionExceptionList]));
  });

  it('fetches the rule with the given ruleId', async () => {
    render();
    await waitFor(() => {
      expect(fetchRuleById).toHaveBeenCalledTimes(1);
      expect(fetchRuleById).toHaveBeenCalledWith({
        id: ruleId,
        signal: abortCtrl.signal,
      });
    });
  });

  describe('when the rule does not have exception list references', () => {
    beforeEach(() => {
      fetchRuleById = jest
        .spyOn(rulesApi, 'fetchRuleById')
        .mockResolvedValue(ruleWithoutExceptionLists);
    });

    it('does not fetch the exceptions lists', async () => {
      render();
      await waitFor(() => expect(fetchExceptionListById).not.toHaveBeenCalled());
    });

    it('should create a new exception list', async () => {
      render();
      await waitFor(() => expect(addExceptionList).toHaveBeenCalledTimes(1));
    });
    it('should update the rule', async () => {
      render();
      await waitFor(() => expect(patchRule).toHaveBeenCalledTimes(1));
    });
    it('invokes onSuccess', async () => {
      render();
      await waitFor(() => expect(onSuccess).toHaveBeenCalledWith(false));
    });
  });

  describe("when the rule has exception list references and 'detection' is passed in", () => {
    it('fetches the exceptions lists', async () => {
      render();
      await waitFor(() => expect(fetchExceptionListById).toHaveBeenCalledTimes(2));
    });

    it('does not create a new exception list', async () => {
      render();
      await waitFor(() => expect(addExceptionList).not.toHaveBeenCalled());
    });

    it('does not update the rule', async () => {
      render();
      await waitFor(() => expect(patchRule).not.toHaveBeenCalled());
    });

    it('should set the exception list to be the fetched list', async () => {
      const { result } = render();
      await waitFor(() => expect(result.current[1]).toEqual(detectionExceptionList));
    });

    it('invokes onSuccess indicating', async () => {
      render();
      await waitFor(() => expect(onSuccess).toHaveBeenCalledWith(false));
    });

    describe("but the rule does not have a reference to 'detection' type exception list", () => {
      beforeEach(() => {
        fetchExceptionListById = jest
          .spyOn(listsApi, 'fetchExceptionListById')
          .mockResolvedValue(endpointExceptionList);
      });

      it('should create a new exception list', async () => {
        render();
        await waitFor(() => expect(addExceptionList).toHaveBeenCalledTimes(1));
      });
      it('should update the rule', async () => {
        render();
        await waitFor(() => expect(patchRule).toHaveBeenCalledTimes(1));
      });
      it('should set the exception list to be the newly created list', async () => {
        const { result } = render();
        await waitFor(() => expect(result.current[1]).toEqual(newDetectionExceptionList));
      });
    });
  });

  describe("when the rule has exception list references and 'endpoint' is passed in", () => {
    beforeEach(() => {
      fetchExceptionListById = jest
        .spyOn(listsApi, 'fetchExceptionListById')
        .mockResolvedValue(endpointExceptionList);

      addExceptionList = jest
        .spyOn(listsApi, 'addExceptionList')
        .mockResolvedValue(newEndpointExceptionList);
    });

    it('fetches the exceptions lists', async () => {
      render(endpointListType);
      await waitFor(() => expect(fetchExceptionListById).toHaveBeenCalledTimes(2));
    });
    it('does not create a new exception list', async () => {
      render(endpointListType);
      await waitFor(() => expect(addExceptionList).not.toHaveBeenCalled());
    });
    it('does not update the rule', async () => {
      render(endpointListType);
      await waitFor(() => expect(patchRule).not.toHaveBeenCalled());
    });
    it('should set the exception list to be the fetched list', async () => {
      const { result } = render(endpointListType);
      await waitFor(() => expect(result.current[1]).toEqual(endpointExceptionList));
    });

    describe("but the rule does not have a reference to 'endpoint' type exception list", () => {
      beforeEach(() => {
        fetchExceptionListById = jest
          .spyOn(listsApi, 'fetchExceptionListById')
          .mockResolvedValue(detectionExceptionList);
      });

      it('should create a new exception list', async () => {
        render(endpointListType);
        await waitFor(() => expect(addEndpointExceptionList).toHaveBeenCalledTimes(1));
      });
      it('should update the rule', async () => {
        render(endpointListType);
        await waitFor(() => expect(patchRule).toHaveBeenCalledTimes(1));
      });
      it('should set the exception list to be the newly created list', async () => {
        const { result } = render(endpointListType);
        await waitFor(() => expect(result.current[1]).toEqual(newEndpointExceptionList));
      });
    });
  });

  describe('when rule api returns an error', () => {
    beforeEach(() => {
      fetchRuleById = jest.spyOn(rulesApi, 'fetchRuleById').mockRejectedValue(error);
    });

    it('exception list should be null', async () => {
      const { result } = render();
      await waitFor(() => expect(result.current[1]).toBeNull());
    });

    it('isLoading should be false', async () => {
      const { result } = render();
      await waitFor(() => expect(result.current[0]).toEqual(false));
    });

    it('should call error callback', async () => {
      render();
      await waitFor(() => {
        expect(onError).toHaveBeenCalledTimes(1);
        expect(onError).toHaveBeenCalledWith(error, null, null);
      });
    });

    it('does not call onSuccess', async () => {
      render();
      await waitFor(() => expect(onSuccess).not.toHaveBeenCalled());
    });
  });
});
