/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import type { IKibanaResponse } from '@kbn/core/server';
import { transformError } from '@kbn/securitysolution-es-utils';
import { buildRouteValidationWithZod } from '@kbn/zod-helpers';
import type { DeleteRuleResponse } from '../../../../../../../common/api/detection_engine/rule_management';
import {
  DeleteRuleRequestQuery,
  validateQueryRuleByIds,
} from '../../../../../../../common/api/detection_engine/rule_management';
import { DETECTION_ENGINE_RULES_URL } from '../../../../../../../common/constants';
import type { SecuritySolutionPluginRouter } from '../../../../../../types';
import { buildSiemResponse } from '../../../../routes/utils';
import { readRules } from '../../../logic/detection_rules_client/read_rules';
import { getIdError, transform } from '../../../utils/utils';

export const deleteRuleRoute = (router: SecuritySolutionPluginRouter) => {
  router.versioned
    .delete({
      access: 'public',
      path: DETECTION_ENGINE_RULES_URL,
      security: {
        authz: {
          requiredPrivileges: ['securitySolution'],
        },
      },
    })
    .addVersion(
      {
        version: '2023-10-31',
        validate: {
          request: {
            query: buildRouteValidationWithZod(DeleteRuleRequestQuery),
          },
        },
      },
      async (context, request, response): Promise<IKibanaResponse<DeleteRuleResponse>> => {
        const siemResponse = buildSiemResponse(response);
        const validationErrors = validateQueryRuleByIds(request.query);
        if (validationErrors.length) {
          return siemResponse.error({ statusCode: 400, body: validationErrors });
        }

        try {
          const { id, rule_id: ruleId } = request.query;

          const ctx = await context.resolve(['core', 'securitySolution', 'alerting']);
          const rulesClient = await ctx.alerting.getRulesClient();
          const detectionRulesClient = ctx.securitySolution.getDetectionRulesClient();

          const rule = await readRules({ rulesClient, id, ruleId });

          if (!rule) {
            const error = getIdError({ id, ruleId });
            return siemResponse.error({
              body: error.message,
              statusCode: error.statusCode,
            });
          }

          await detectionRulesClient.deleteRule({
            ruleId: rule.id,
          });

          const transformed = transform(rule);
          if (transformed == null) {
            return siemResponse.error({ statusCode: 500, body: 'failed to transform alert' });
          } else {
            return response.ok({ body: transformed ?? {} });
          }
        } catch (err) {
          const error = transformError(err);
          return siemResponse.error({
            body: error.message,
            statusCode: error.statusCode,
          });
        }
      }
    );
};
