/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { getThreatMock } from '../../../../../common/detection_engine/schemas/types/threat.mock';
import { getListArrayMock } from '../../../../../common/detection_engine/schemas/types/lists.mock';
import { getThreatMappingMock } from '../../rule_types/indicator_match/threat_mapping/build_threat_mapping_filter.mock';
import type {
  BaseRuleParams,
  CompleteRule,
  EqlRuleParams,
  EsqlRuleParams,
  MachineLearningRuleParams,
  NewTermsRuleParams,
  QueryRuleParams,
  RuleParams,
  SavedQueryRuleParams,
  ThreatRuleParams,
  ThresholdRuleParams,
} from '..';
import type { SanitizedRuleConfig } from '@kbn/alerting-plugin/common';
import { sampleRuleGuid } from '../../rule_types/__mocks__/es_results';

export const getBaseRuleParams = (): BaseRuleParams => {
  return {
    author: ['Elastic'],
    buildingBlockType: 'default',
    ruleId: 'rule-1',
    description: 'Detecting root and admin users',
    falsePositives: [],
    immutable: false,
    ruleSource: {
      type: 'internal',
    },
    from: 'now-6m',
    to: 'now',
    severity: 'high',
    severityMapping: [],
    license: 'Elastic License',
    outputIndex: '.siem-signals',
    references: ['http://example.com', 'https://example.com'],
    riskScore: 50,
    riskScoreMapping: [],
    ruleNameOverride: undefined,
    maxSignals: 10000,
    namespace: undefined,
    note: '# Investigative notes',
    timelineId: 'some-timeline-id',
    timelineTitle: 'some-timeline-title',
    timestampOverride: undefined,
    timestampOverrideFallbackDisabled: undefined,
    meta: {
      someMeta: 'someField',
    },
    threat: getThreatMock(),
    version: 1,
    exceptionsList: getListArrayMock(),
    relatedIntegrations: [],
    requiredFields: [],
    setup: '',
    investigationFields: undefined,
  };
};

export const getThresholdRuleParams = (
  rewrites?: Partial<ThresholdRuleParams>
): ThresholdRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'threshold',
    language: 'kuery',
    index: ['auditbeat-*', 'filebeat-*', 'packetbeat-*', 'winlogbeat-*'],
    dataViewId: undefined,
    query: 'user.name: root or user.name: admin',
    filters: undefined,
    savedId: undefined,
    threshold: {
      field: ['host.id'],
      value: 5,
      cardinality: [
        {
          field: 'source.ip',
          value: 11,
        },
      ],
    },
    ...rewrites,
  };
};

export const getEqlRuleParams = (rewrites?: Partial<EqlRuleParams>): EqlRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'eql',
    language: 'eql',
    index: ['some-index'],
    query: 'any where true',
    filters: undefined,
    timestampField: undefined,
    eventCategoryOverride: undefined,
    dataViewId: undefined,
    tiebreakerField: undefined,
    ...rewrites,
  };
};

export const getEsqlRuleParams = (rewrites?: Partial<EsqlRuleParams>): EsqlRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'esql',
    language: 'esql',
    query: 'from auditbeat* metadata _id',
    ...rewrites,
  };
};

export const getMlRuleParams = (
  rewrites?: Partial<MachineLearningRuleParams>
): MachineLearningRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'machine_learning',
    anomalyThreshold: 42,
    machineLearningJobId: ['my-job'],
    ...rewrites,
  };
};

export const getQueryRuleParams = (rewrites?: Partial<QueryRuleParams>): QueryRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'query',
    language: 'kuery',
    query: 'user.name: root or user.name: admin',
    index: ['auditbeat-*', 'filebeat-*', 'packetbeat-*', 'winlogbeat-*'],
    dataViewId: undefined,
    filters: [
      {
        query: {
          match_phrase: {
            'host.name': 'some-host',
          },
        },
      },
    ],
    savedId: undefined,
    alertSuppression: undefined,
    responseActions: undefined,
    ...rewrites,
  };
};

export const getSavedQueryRuleParams = (
  rewrites?: Partial<SavedQueryRuleParams>
): SavedQueryRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'saved_query',
    language: 'kuery',
    query: 'user.name: root or user.name: admin',
    index: ['auditbeat-*', 'filebeat-*', 'packetbeat-*', 'winlogbeat-*'],
    dataViewId: undefined,
    filters: [
      {
        query: {
          match_phrase: {
            'host.name': 'some-host',
          },
        },
      },
    ],
    savedId: 'some-id',
    responseActions: undefined,
    alertSuppression: undefined,
    ...rewrites,
  };
};

export const getNewTermsRuleParams = (
  rewrites?: Partial<NewTermsRuleParams>
): NewTermsRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'new_terms',
    language: 'kuery',
    query: 'user.name: root or user.name: admin',
    index: ['auditbeat-*', 'filebeat-*', 'packetbeat-*', 'winlogbeat-*'],
    dataViewId: undefined,
    filters: [
      {
        query: {
          match_phrase: {
            'host.name': 'some-host',
          },
        },
      },
    ],
    newTermsFields: ['host.name'],
    historyWindowStart: 'now-30d',
    ...rewrites,
  };
};

export const getThreatRuleParams = (rewrites?: Partial<ThreatRuleParams>): ThreatRuleParams => {
  return {
    ...getBaseRuleParams(),
    type: 'threat_match',
    language: 'kuery',
    query: '*:*',
    index: ['some-index'],
    dataViewId: undefined,
    filters: undefined,
    savedId: undefined,
    threatQuery: 'threat-query',
    threatFilters: undefined,
    threatIndex: ['some-threat-index'],
    threatLanguage: 'kuery',
    threatMapping: getThreatMappingMock(),
    threatIndicatorPath: '',
    concurrentSearches: undefined,
    itemsPerSearch: undefined,
    ...rewrites,
  };
};

export const getRuleConfigMock = (type: string = 'rule-type'): SanitizedRuleConfig => ({
  id: sampleRuleGuid,
  actions: [],
  enabled: true,
  name: 'rule-name',
  tags: ['some fake tag 1', 'some fake tag 2'],
  createdBy: 'sample user',
  createdAt: new Date('2020-03-27T22:55:59.577Z'),
  updatedAt: new Date('2020-03-27T22:55:59.577Z'),
  updatedBy: 'sample user',
  schedule: {
    interval: '5m',
  },
  throttle: 'no_actions',
  consumer: 'sample consumer',
  notifyWhen: null,
  producer: 'sample producer',
  revision: 0,
  ruleTypeId: `${type}-id`,
  ruleTypeName: type,
  muteAll: false,
  snoozeSchedule: [],
});

export const getCompleteRuleMock = <T extends RuleParams>(params: T): CompleteRule<T> => ({
  alertId: sampleRuleGuid,
  ruleParams: params,
  ruleConfig: getRuleConfigMock(),
});
