/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import { ENDPOINT_ARTIFACT_LISTS } from '@kbn/securitysolution-list-constants';
import type { TypeOf } from '@kbn/config-schema';
import { schema } from '@kbn/config-schema';
import type { ExceptionListItemSchema } from '@kbn/securitysolution-io-ts-list-types';
import type { TrustedAppEntryTypes } from '@kbn/securitysolution-utils';
import { OperatingSystem } from '@kbn/securitysolution-utils';
import type {
  CreateExceptionListItemOptions,
  UpdateExceptionListItemOptions,
} from '@kbn/lists-plugin/server';
import { hasArtifactOwnerSpaceId } from '../../../../common/endpoint/service/artifacts/utils';
import { BaseValidator } from './base_validator';
import type { ExceptionItemLikeOptions } from '../types';
import type { TrustedAppConditionEntry as ConditionEntry } from '../../../../common/endpoint/types';
import {
  getDuplicateFields,
  isValidHash,
} from '../../../../common/endpoint/service/artifacts/validations';
import { EndpointArtifactExceptionValidationError } from './errors';

const ProcessHashField = schema.oneOf([
  schema.literal('process.hash.md5'),
  schema.literal('process.hash.sha1'),
  schema.literal('process.hash.sha256'),
]);
const ProcessExecutablePath = schema.literal('process.executable.caseless');
const ProcessWindowsCodeSigner = schema.literal('process.Ext.code_signature');
const ProcessMacCodeSigner = schema.literal('process.code_signature');

const ConditionEntryTypeSchema = schema.conditional(
  schema.siblingRef('field'),
  ProcessExecutablePath,
  schema.oneOf([schema.literal('match'), schema.literal('wildcard')]),
  schema.literal('match')
);
const ConditionEntryOperatorSchema = schema.literal('included');

type ConditionEntryFieldAllowedType =
  | TypeOf<typeof ProcessHashField>
  | TypeOf<typeof ProcessExecutablePath>
  | TypeOf<typeof ProcessWindowsCodeSigner>
  | TypeOf<typeof ProcessMacCodeSigner>;

type TrustedAppConditionEntry<
  T extends ConditionEntryFieldAllowedType = ConditionEntryFieldAllowedType
> =
  | {
      field: T;
      type: TrustedAppEntryTypes;
      operator: 'included';
      value: string;
    }
  | TypeOf<typeof SignerWindowsEntrySchema>
  | TypeOf<typeof SignerMacEntrySchema>;

/*
 * A generic Entry schema to be used for a specific entry schema depending on the OS
 */
const CommonEntrySchema = {
  field: schema.oneOf([ProcessHashField, ProcessExecutablePath]),
  type: ConditionEntryTypeSchema,
  operator: ConditionEntryOperatorSchema,
  // If field === HASH then validate hash with custom method, else validate string with minLength = 1
  value: schema.conditional(
    schema.siblingRef('field'),
    ProcessHashField,
    schema.string({
      validate: (hash: string) => (isValidHash(hash) ? undefined : `invalid hash value [${hash}]`),
    }),
    schema.conditional(
      schema.siblingRef('field'),
      ProcessExecutablePath,
      schema.string({
        validate: (pathValue: string) =>
          pathValue.length > 0 ? undefined : `invalid path value [${pathValue}]`,
      }),
      schema.string({
        validate: (signerValue: string) =>
          signerValue.length > 0 ? undefined : `invalid signer value [${signerValue}]`,
      })
    )
  ),
};

// Windows/MacOS Signer entries use a Nested field that checks to ensure
// that the certificate is trusted
const SignerEntrySchema = {
  type: schema.literal('nested'),
  entries: schema.arrayOf(
    schema.oneOf([
      schema.object({
        field: schema.literal('trusted'),
        value: schema.literal('true'),
        type: schema.literal('match'),
        operator: schema.literal('included'),
      }),
      schema.object({
        field: schema.literal('subject_name'),
        value: schema.string({ minLength: 1 }),
        type: schema.literal('match'),
        operator: schema.literal('included'),
      }),
    ]),
    { minSize: 2, maxSize: 2 }
  ),
};

const SignerWindowsEntrySchema = schema.object({
  ...SignerEntrySchema,
  field: ProcessWindowsCodeSigner,
});

const SignerMacEntrySchema = schema.object({
  ...SignerEntrySchema,
  field: ProcessMacCodeSigner,
});

const WindowsEntrySchema = schema.oneOf([
  SignerWindowsEntrySchema,
  schema.object({
    ...CommonEntrySchema,
    field: schema.oneOf([ProcessHashField, ProcessExecutablePath]),
  }),
]);

const MacEntrySchema = schema.oneOf([
  SignerMacEntrySchema,
  schema.object({
    ...CommonEntrySchema,
    field: schema.oneOf([ProcessHashField, ProcessExecutablePath]),
  }),
]);

const LinuxEntrySchema = schema.object({
  ...CommonEntrySchema,
});

const entriesSchemaOptions = {
  minSize: 1,
  validate(entries: TrustedAppConditionEntry[]) {
    const dups = getDuplicateFields(entries as ConditionEntry[]);
    return dups.map((field) => `Duplicated entry: ${field}`).join(', ') || undefined;
  },
};

/*
 * Entities array schema depending on Os type using schema.conditional.
 * If OS === WINDOWS then use Windows schema,
 * else if OS === LINUX then use Linux schema,
 * else use Mac schema
 *
 * The validate function checks there is no duplicated entry inside the array
 */
const EntriesSchema = schema.conditional(
  schema.contextRef('os'),
  OperatingSystem.WINDOWS,
  schema.arrayOf(WindowsEntrySchema, entriesSchemaOptions),
  schema.conditional(
    schema.contextRef('os'),
    OperatingSystem.LINUX,
    schema.arrayOf(LinuxEntrySchema, entriesSchemaOptions),
    schema.arrayOf(MacEntrySchema, entriesSchemaOptions)
  )
);

/**
 * Schema to validate Trusted Apps data for create and update.
 * When called, it must be given an `context` with a `os` property set
 *
 * @example
 *
 * TrustedAppDataSchema.validate(item, { os: 'windows' });
 */
const TrustedAppDataSchema = schema.object(
  {
    entries: EntriesSchema,
  },

  // Because we are only validating some fields from the Exception Item, we set `unknowns` to `ignore` here
  { unknowns: 'ignore' }
);

export class TrustedAppValidator extends BaseValidator {
  static isTrustedApp(item: { listId: string }): boolean {
    return item.listId === ENDPOINT_ARTIFACT_LISTS.trustedApps.id;
  }

  protected async validateHasWritePrivilege(): Promise<void> {
    return super.validateHasPrivilege('canWriteTrustedApplications');
  }

  protected async validateHasReadPrivilege(): Promise<void> {
    return super.validateHasPrivilege('canReadTrustedApplications');
  }

  async validatePreCreateItem(
    item: CreateExceptionListItemOptions
  ): Promise<CreateExceptionListItemOptions> {
    await this.validateHasWritePrivilege();
    await this.validateTrustedAppData(item);
    await this.validateCanCreateByPolicyArtifacts(item);
    await this.validateByPolicyItem(item);
    await this.validateCreateOwnerSpaceIds(item);
    await this.validateCanCreateGlobalArtifacts(item);

    await this.setOwnerSpaceId(item);

    return item;
  }

  async validatePreDeleteItem(currentItem: ExceptionListItemSchema): Promise<void> {
    await this.validateHasWritePrivilege();
    await this.validateCanDeleteItemInActiveSpace(currentItem);
  }

  async validatePreGetOneItem(): Promise<void> {
    await this.validateHasReadPrivilege();
  }

  async validatePreMultiListFind(): Promise<void> {
    await this.validateHasReadPrivilege();
  }

  async validatePreExport(): Promise<void> {
    await this.validateHasReadPrivilege();
  }

  async validatePreSingleListFind(): Promise<void> {
    await this.validateHasReadPrivilege();
  }

  async validatePreGetListSummary(): Promise<void> {
    await this.validateHasReadPrivilege();
  }

  async validatePreUpdateItem(
    _updatedItem: UpdateExceptionListItemOptions,
    currentItem: ExceptionListItemSchema
  ): Promise<UpdateExceptionListItemOptions> {
    const updatedItem = _updatedItem as ExceptionItemLikeOptions;

    await this.validateHasWritePrivilege();
    await this.validateTrustedAppData(updatedItem);

    try {
      await this.validateCanCreateByPolicyArtifacts(updatedItem);
    } catch (noByPolicyAuthzError) {
      // Not allowed to create/update by policy data. Validate that the effective scope of the item
      // remained unchanged with this update or was set to `global` (only allowed update). If not,
      // then throw the validation error that was catch'ed
      if (this.wasByPolicyEffectScopeChanged(updatedItem, currentItem)) {
        throw noByPolicyAuthzError;
      }
    }

    await this.validateByPolicyItem(updatedItem);
    await this.validateUpdateOwnerSpaceIds(_updatedItem, currentItem);
    await this.validateCanUpdateItemInActiveSpace(_updatedItem, currentItem);

    if (!hasArtifactOwnerSpaceId(_updatedItem)) {
      await this.setOwnerSpaceId(_updatedItem);
    }

    return _updatedItem;
  }

  private async validateTrustedAppData(item: ExceptionItemLikeOptions): Promise<void> {
    await this.validateBasicData(item);

    try {
      TrustedAppDataSchema.validate(item, { os: item.osTypes[0] });
    } catch (error) {
      throw new EndpointArtifactExceptionValidationError(error.message);
    }
  }
}
