/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React, { memo, ReactNode, useMemo } from 'react';
import { createMemoryHistory, MemoryHistory } from 'history';
import { render as reactRender, RenderOptions, RenderResult } from '@testing-library/react';
import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import { Router } from '@kbn/shared-ux-router';
import { History } from 'history';
import { I18nProvider } from '@kbn/i18n-react';
import { CoreStart } from '@kbn/core/public';
import { coreMock } from '@kbn/core/public/mocks';
import { KibanaContextProvider } from '@kbn/kibana-react-plugin/public';
import { EuiThemeProvider } from '@kbn/kibana-react-plugin/common';
import { SECURITY_APP_ID, SESSION_VIEW_APP_ID } from '../../common/constants';

type UiRender = (ui: React.ReactElement, options?: RenderOptions) => RenderResult;

/**
 * Mocked app root context renderer
 */
export interface AppContextTestRender {
  history: ReturnType<typeof createMemoryHistory>;
  coreStart: ReturnType<typeof coreMock.createStart>;
  /**
   * A wrapper around `AppRootContext` component. Uses the mocked modules as input to the
   * `AppRootContext`
   */
  AppWrapper: React.FC<any>;
  /**
   * Renders the given UI within the created `AppWrapper` providing the given UI a mocked
   * endpoint runtime context environment
   */
  render: UiRender;
}

const createCoreStartMock = (
  history: MemoryHistory<never>
): ReturnType<typeof coreMock.createStart> => {
  const coreStart = coreMock.createStart({ basePath: '/mock' });

  // Mock the certain APP Ids returned by `application.getUrlForApp()`
  coreStart.application.getUrlForApp.mockImplementation((appId) => {
    switch (appId) {
      case SESSION_VIEW_APP_ID:
        return '/app/sessionView';
      case SECURITY_APP_ID:
        return '/app/security';
      default:
        return `${appId} not mocked!`;
    }
  });

  coreStart.application.navigateToUrl.mockImplementation((url) => {
    history.push(url.replace('/app/sessionView', ''));
    return Promise.resolve();
  });

  return coreStart;
};

const AppRootProvider = memo<{
  history: History;
  coreStart: CoreStart;
  children: ReactNode | ReactNode[];
}>(({ history, coreStart: { http, notifications, theme, application }, children }) => {
  const isDarkMode = theme.getTheme().darkMode;
  const services = useMemo(
    () => ({ http, notifications, application }),
    [application, http, notifications]
  );
  return (
    <I18nProvider>
      <KibanaContextProvider services={services}>
        <EuiThemeProvider darkMode={isDarkMode}>
          <Router history={history}>{children}</Router>
        </EuiThemeProvider>
      </KibanaContextProvider>
    </I18nProvider>
  );
});

AppRootProvider.displayName = 'AppRootProvider';

/**
 * Creates a mocked app context custom renderer that can be used to render
 * component that depend upon the application's surrounding context providers.
 * Factory also returns the content that was used to create the custom renderer, allowing
 * for further customization.
 */

export const createAppRootMockRenderer = (): AppContextTestRender => {
  const history = createMemoryHistory<never>();
  const coreStart = createCoreStartMock(history);

  const queryClient = new QueryClient({
    defaultOptions: {
      queries: {
        // turns retries off
        retry: false,
        // prevent jest did not exit errors
        cacheTime: Infinity,
      },
    },
    // hide react-query output in console
    logger: {
      error: () => {},
      // eslint-disable-next-line no-console
      log: console.log,
      // eslint-disable-next-line no-console
      warn: console.warn,
    },
  });

  const AppWrapper: React.FC<{ children: React.ReactNode }> = ({ children }) => (
    <AppRootProvider history={history} coreStart={coreStart}>
      <QueryClientProvider client={queryClient}>{children}</QueryClientProvider>
    </AppRootProvider>
  );

  const render: UiRender = (ui, options = {}) => {
    return reactRender(ui, {
      wrapper: AppWrapper,
      ...options,
    });
  };

  return {
    history,
    coreStart,
    AppWrapper,
    render,
  };
};
