/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import {
  expectIsViewOnly,
  getPolicySettingsFormTestSubjects,
  exactMatchText,
  setMalwareMode,
} from '../../mocks';
import type { AppContextTestRender } from '../../../../../../../common/mock/endpoint';
import { createAppRootMockRenderer } from '../../../../../../../common/mock/endpoint';
import { FleetPackagePolicyGenerator } from '../../../../../../../../common/endpoint/data_generators/fleet_package_policy_generator';
import React from 'react';
import type { MalwareProtectionsProps } from './malware_protections_card';
import { MalwareProtectionsCard } from './malware_protections_card';
import type { PolicyConfig } from '../../../../../../../../common/endpoint/types';
import { ProtectionModes } from '../../../../../../../../common/endpoint/types';
import { cloneDeep } from 'lodash';
import { set } from '@kbn/safer-lodash-set';
import userEvent from '@testing-library/user-event';

jest.mock('../../../../../../../common/hooks/use_license');

describe('Policy Malware Protections Card', () => {
  const testSubj = getPolicySettingsFormTestSubjects('test').malware;

  let formProps: MalwareProtectionsProps;
  let render: (policyConfig?: PolicyConfig) => ReturnType<AppContextTestRender['render']>;
  let renderResult: ReturnType<typeof render>;

  beforeEach(() => {
    const mockedContext = createAppRootMockRenderer();
    mockedContext.setExperimentalFlag({ malwareOnWriteScanOptionAvailable: true });

    formProps = {
      policy: new FleetPackagePolicyGenerator('seed').generateEndpointPackagePolicy().inputs[0]
        .config.policy.value,
      onChange: jest.fn(),
      mode: 'edit',
      'data-test-subj': testSubj.card,
    };

    render = (policyConfig = formProps.policy) =>
      (renderResult = mockedContext.render(
        <MalwareProtectionsCard {...formProps} policy={policyConfig} />
      ));
  });

  it('should render the card with expected components', () => {
    const { getByTestId } = render();

    expect(getByTestId(testSubj.enableDisableSwitch));
    expect(getByTestId(testSubj.protectionPreventRadio));
    expect(getByTestId(testSubj.notifyUserCheckbox));
  });

  it('should show supported OS values', () => {
    render();

    expect(renderResult.getByTestId(testSubj.osValuesContainer)).toHaveTextContent(
      'Windows, Mac, Linux'
    );
  });

  describe.each`
    name             | config             | default
    ${'blocklist'}   | ${'blocklist'}     | ${true}
    ${'onWriteScan'} | ${'on_write_scan'} | ${true}
  `(
    '$name subfeature',
    (feature: { name: 'blocklist' | 'onWriteScan'; config: string; deafult: boolean }) => {
      it(`should set ${feature.name} to disabled  if malware is turned off`, async () => {
        const expectedUpdatedPolicy = cloneDeep(formProps.policy);
        setMalwareMode({ policy: expectedUpdatedPolicy, turnOff: true });
        render();
        await userEvent.click(renderResult.getByTestId(testSubj.enableDisableSwitch));

        expect(formProps.onChange).toHaveBeenCalledWith({
          isValid: true,
          updatedPolicy: expectedUpdatedPolicy,
        });
      });

      it(`should set ${feature.name} to enabled if malware is turned on`, async () => {
        const expectedUpdatedPolicy = cloneDeep(formProps.policy);
        setMalwareMode({ policy: expectedUpdatedPolicy });
        const initialPolicy = cloneDeep(formProps.policy);
        setMalwareMode({ policy: initialPolicy, turnOff: true });
        render(initialPolicy);

        await userEvent.click(renderResult.getByTestId(testSubj.enableDisableSwitch));

        expect(formProps.onChange).toHaveBeenCalledWith({
          isValid: true,
          updatedPolicy: expectedUpdatedPolicy,
        });
      });

      it(`should allow ${feature.name} to be disabled`, async () => {
        const expectedUpdatedPolicy = cloneDeep(formProps.policy);
        set(expectedUpdatedPolicy, `windows.malware.${feature.config}`, false);
        set(expectedUpdatedPolicy, `mac.malware.${feature.config}`, false);
        set(expectedUpdatedPolicy, `linux.malware.${feature.config}`, false);
        render();
        await userEvent.click(
          renderResult.getByTestId(testSubj[`${feature.name}EnableDisableSwitch`])
        );

        expect(formProps.onChange).toHaveBeenCalledWith({
          isValid: true,
          updatedPolicy: expectedUpdatedPolicy,
        });
      });

      it(`should allow ${feature.name} to be enabled`, async () => {
        set(formProps.policy, `windows.malware.${feature.config}`, false);
        set(formProps.policy, `mac.malware.${feature.config}`, false);
        set(formProps.policy, `linux.malware.${feature.config}`, false);
        const expectedUpdatedPolicy = cloneDeep(formProps.policy);
        set(expectedUpdatedPolicy, `windows.malware.${feature.config}`, true);
        set(expectedUpdatedPolicy, `mac.malware.${feature.config}`, true);
        set(expectedUpdatedPolicy, `linux.malware.${feature.config}`, true);
        render();
        await userEvent.click(
          renderResult.getByTestId(testSubj[`${feature.name}EnableDisableSwitch`])
        );

        expect(formProps.onChange).toHaveBeenCalledWith({
          isValid: true,
          updatedPolicy: expectedUpdatedPolicy,
        });
      });
    }
  );

  describe('and displayed in View mode', () => {
    beforeEach(() => {
      formProps.mode = 'view';
    });

    it('should display correctly when overall card is enabled', () => {
      const { getByTestId } = render();

      expectIsViewOnly(getByTestId(testSubj.card));

      expect(getByTestId(testSubj.card)).toHaveTextContent(
        exactMatchText(
          'Type' +
            'Malware' +
            'Operating system' +
            'Windows, Mac, Linux ' +
            'Malware protections' +
            'Protection level' +
            'Prevent' +
            'Blocklist' +
            'Info' +
            'Scan files upon modification' +
            'Info' +
            'User notification' +
            'Agent version 7.11+' +
            'Notify user' +
            'Notification message' +
            '—'
        )
      );
    });

    it('should display correctly when overall card is disabled', () => {
      set(formProps.policy, 'windows.malware.mode', ProtectionModes.off);
      const { getByTestId } = render();

      expectIsViewOnly(getByTestId(testSubj.card));

      expect(getByTestId(testSubj.card)).toHaveTextContent(
        exactMatchText(
          [
            'Type',
            'Malware',
            'Operating system',
            'Windows, Mac, Linux ',
            'Malware protections',
          ].join('')
        )
      );
    });

    it('should display user notification disabled', () => {
      set(formProps.policy, 'windows.popup.malware.enabled', false);

      const { getByTestId } = render();

      expectIsViewOnly(getByTestId(testSubj.card));

      expect(getByTestId(testSubj.card)).toHaveTextContent(
        exactMatchText(
          'Type' +
            'Malware' +
            'Operating system' +
            'Windows, Mac, Linux ' +
            'Malware protections' +
            'Protection level' +
            'Prevent' +
            'Blocklist' +
            'Info' +
            'Scan files upon modification' +
            'Info' +
            'User notification' +
            'Agent version 7.11+' +
            'Notify user'
        )
      );
    });
  });
});
